var animations = animations || {};

animations.animateTower = function( Tower , type , shape ){

	// Here we will animate the tower shotting by type
	switch( type ){
		
		case 'tetra':
			animateTetra( Tower , shape );
			break;

		case 'cube':
			animateCube( Tower , shape );
			break;
	}


}

animations.animationPresets = function( Tower , type ){

	// Here we will process the animation.  To make things simple, we're going to have a different animation cycle for each tower type.
	switch( type ){

		case 'tetra':
			// determine which barrel is set to fire
			// since we want to alternate between barrels we'll just switch back and forth 
			// from barrel 0 to 1 to 0 to 1 etc...
			Tower.soundFX = new Audio( "sounds/tetraShot.wav" );
			if( Tower.details.barrelToFire == 0 ){
				Tower.details.barrelToFire ++;
				Tower.details.animating = true;
				Tower.details.rest = 0;
			} else {
				Tower.details.barrelToFire = 0;
				Tower.details.animating = true;
				Tower.details.rest = 0;
			}
			break;

		case 'cube':
			Tower.soundFX = new Audio( "sounds/cubeCannon.wav" );
			Tower.details.animating = true;
			Tower.details.rest = 0;
			break;
	}
}

function animateTetra( Tower , shape ){

	// now we'll check the animation cycle
	switch( Tower.details.rest ){
		case 1:
			// we'll get the position of the turret and the direction it's pointing in. The position we're looking for is the real
			// world position, not the local one to the tower.
			Tower.updateMatrixWorld();

			// both World variables are the world positions of the two vertices.  The vertices are taken from local space,
			// which is their relative position within the model, then the models scale and position is taken into consideration
			// and the position is updated accordingly.
			var barrelStartWorld = Tower.geometry.vertices[ Tower.geometry.vertices.length - Tower.details.barrelToFire - 1 ].clone();
			Tower.barrelEndWorld = Tower.geometry.vertices[ Tower.geometry.vertices.length - Tower.details.barrelToFire - 3 ].clone();
			
			// both local variables are the positions of the vertices relative to the model, not to the world.
			barrelStartLocal = Tower.geometry.vertices[ Tower.geometry.vertices.length - Tower.details.barrelToFire - 1 ].clone();
			barrelEndLocal = Tower.geometry.vertices[ Tower.geometry.vertices.length - Tower.details.barrelToFire - 3 ].clone();

			// transform the vertice from local to world co-ordinates.
			barrelStartWorld.applyMatrix4( Tower.matrixWorld );
			Tower.barrelEndWorld.applyMatrix4( Tower.matrixWorld );

			// we're going to create two vectors reprenting the directions from one end of the barrel to the other
			// to animate the recoil of the barrel we need to know the offset of the vertices within local co-ordinates, not world.
			Tower.barrelOffset = barrelEndLocal.clone().sub( barrelStartLocal ).normalize();
			// to animate the 3D fire burst we need to know  the location of the barrel's end and the direction the barrel is facing
			// in world co-ordinates.
			Tower.barrelOffsetDirection = Tower.barrelEndWorld.clone().sub( barrelStartWorld ).normalize();

			// since offset has been normalized it's distance is effectively 1, so we'll soften the distance of the recoil by 
			// dividing it in half.  Now the barrelOffset will be a distance of 0.5
			Tower.barrelOffset.multiplyScalar( 0.5 );

			// lookAtTarget will be the lookAt position for the 3D fire burst animations
			Tower.lookAtTarget = Tower.barrelEndWorld.clone().add( Tower.barrelOffsetDirection );

			// 3 vertices for tip of barrels within local space
			Tower.barrelCornerOne = Tower.geometry.vertices[ Tower.geometry.vertices.length - 12 + Tower.details.barrelToFire ];
			Tower.barrelCornerTwo = Tower.geometry.vertices[ Tower.geometry.vertices.length - 15 + ( Tower.details.barrelToFire * 2 ) ];
			Tower.barrelCornerThree = Tower.geometry.vertices[ Tower.geometry.vertices.length - 16 + ( Tower.details.barrelToFire * 2 ) ];

			// We'll update the position of the first frame of the animation to be equal to the end of the barrel in world space.
			Tower.details.animation[ 0 ].position.set( Tower.barrelEndWorld.x , Tower.barrelEndWorld.y , Tower.barrelEndWorld.z );
			Tower.details.animation[ 0 ].lookAt( Tower.lookAtTarget.clone() );
			Tower.details.animation[ 0 ].rotateZ( Math.random() * Math.PI );
			Tower.details.animation[ 0 ].visible = true;

			Tower.soundFX.play();

			break;
		case 2:

			// unload first frame of fire burst animation.
			// load second frame of fire burst animation.
			Tower.details.animation[ 0 ].visible = false;
			Tower.details.animation[ 1 ].position.set( Tower.barrelEndWorld.x , Tower.barrelEndWorld.y , Tower.barrelEndWorld.z );
			Tower.details.animation[ 1 ].lookAt( Tower.lookAtTarget.clone() );
			Tower.details.animation[ 1 ].rotateZ( Math.random() * Math.PI );
			Tower.details.animation[ 1 ].visible = true;

			// begin recoil of barrel in local space.
			Tower.barrelCornerOne.set( Tower.barrelCornerOne.x - ( Tower.barrelOffset.x * 2 ) , Tower.barrelCornerOne.y - ( Tower.barrelOffset.y * 2 ) , Tower.barrelCornerOne.z - ( Tower.barrelOffset.z * 2 ) );
			Tower.barrelCornerTwo.set( Tower.barrelCornerTwo.x - ( Tower.barrelOffset.x * 2 ) , Tower.barrelCornerTwo.y - ( Tower.barrelOffset.y * 2 ) , Tower.barrelCornerTwo.z - ( Tower.barrelOffset.z * 2 ) );
			Tower.barrelCornerThree.set( Tower.barrelCornerThree.x - ( Tower.barrelOffset.x * 2 ) , Tower.barrelCornerThree.y - ( Tower.barrelOffset.y * 2 ) , Tower.barrelCornerThree.z - ( Tower.barrelOffset.z * 2 ) );
			Tower.geometry.verticesNeedUpdate = true;
			break;
		case 3:

			// unload second frame of fire burst animation
			Tower.details.animation[ 1 ].visible = false;

			// continue recoil of barrel in local space.
			Tower.barrelCornerOne.set( Tower.barrelCornerOne.x - Tower.barrelOffset.x , Tower.barrelCornerOne.y - Tower.barrelOffset.y , Tower.barrelCornerOne.z - Tower.barrelOffset.z );
			Tower.barrelCornerTwo.set( Tower.barrelCornerTwo.x - Tower.barrelOffset.x , Tower.barrelCornerTwo.y - Tower.barrelOffset.y , Tower.barrelCornerTwo.z - Tower.barrelOffset.z );
			Tower.barrelCornerThree.set( Tower.barrelCornerThree.x - Tower.barrelOffset.x , Tower.barrelCornerThree.y - Tower.barrelOffset.y , Tower.barrelCornerThree.z - Tower.barrelOffset.z );
			Tower.geometry.verticesNeedUpdate = true;
			break;

		case 4:

			// reset recoil of barrel in local space
			Tower.barrelCornerOne.set( Tower.barrelCornerOne.x + Tower.barrelOffset.x , Tower.barrelCornerOne.y + Tower.barrelOffset.y , Tower.barrelCornerOne.z + Tower.barrelOffset.z );
			Tower.barrelCornerTwo.set( Tower.barrelCornerTwo.x + Tower.barrelOffset.x , Tower.barrelCornerTwo.y + Tower.barrelOffset.y , Tower.barrelCornerTwo.z + Tower.barrelOffset.z );
			Tower.barrelCornerThree.set( Tower.barrelCornerThree.x + Tower.barrelOffset.x , Tower.barrelCornerThree.y + Tower.barrelOffset.y , Tower.barrelCornerThree.z + Tower.barrelOffset.z );
			Tower.geometry.verticesNeedUpdate = true;
			break;

		case 5:

			// reset recoil of barrel in local space
			Tower.barrelCornerOne.set( Tower.barrelCornerOne.x + Tower.barrelOffset.x , Tower.barrelCornerOne.y + Tower.barrelOffset.y , Tower.barrelCornerOne.z + Tower.barrelOffset.z );
			Tower.barrelCornerTwo.set( Tower.barrelCornerTwo.x + Tower.barrelOffset.x , Tower.barrelCornerTwo.y + Tower.barrelOffset.y , Tower.barrelCornerTwo.z + Tower.barrelOffset.z );
			Tower.barrelCornerThree.set( Tower.barrelCornerThree.x + Tower.barrelOffset.x , Tower.barrelCornerThree.y + Tower.barrelOffset.y , Tower.barrelCornerThree.z + Tower.barrelOffset.z );
			Tower.geometry.verticesNeedUpdate = true;								
			break;

		case 6:

			// reset recoil of barrel in local space
			Tower.barrelCornerOne.set( Tower.barrelCornerOne.x + Tower.barrelOffset.x , Tower.barrelCornerOne.y + Tower.barrelOffset.y , Tower.barrelCornerOne.z + Tower.barrelOffset.z );
			Tower.barrelCornerTwo.set( Tower.barrelCornerTwo.x + Tower.barrelOffset.x , Tower.barrelCornerTwo.y + Tower.barrelOffset.y , Tower.barrelCornerTwo.z + Tower.barrelOffset.z );
			Tower.barrelCornerThree.set( Tower.barrelCornerThree.x + Tower.barrelOffset.x , Tower.barrelCornerThree.y + Tower.barrelOffset.y , Tower.barrelCornerThree.z + Tower.barrelOffset.z );
			Tower.geometry.verticesNeedUpdate = true;

			// in this frame we will process the shape's hit
			// in order we'll check for pentagons, triangles and then lastly squares.
			// this order is for the tetra-tosser.
			if( shape.shapes.pentagons > 0 ){

				shape.shapes.pentagons -=3;
				shape.shapes.total -=3;

				var faceOne = shape.geometry.faces[ shape.shapeIndex.pentagons[ 0 ] ];
				var faceTwo = shape.geometry.faces[ shape.shapeIndex.pentagons[ 1 ] ];
				var faceThree = shape.geometry.faces[ shape.shapeIndex.pentagons[ 2 ] ];

				// first we'll preserve the original draw oder
				faceOne.oldA = faceOne.a;	faceTwo.oldA = faceTwo.a;	faceThree.oldA = faceThree.a;
				faceOne.oldB = faceOne.b;	faceTwo.oldB = faceTwo.b;	faceThree.oldB = faceThree.b;
				faceOne.oldC = faceOne.c;	faceTwo.oldC = faceTwo.c;	faceThree.oldC = faceThree.c;

				// reset the vertices draw order
				faceOne.a = 0;	faceTwo.a = 0;	faceThree.a = 0;
				faceOne.b = 0;	faceTwo.b = 0;	faceThree.b = 0;
				faceOne.c = 0;	faceTwo.c = 0;	faceThree.c = 0;

				shape.geometry.elementsNeedUpdate = true;

				// remove these three faces from the pentagons array and add them to
				// the pentagonsToRegernerate array
				shape.shapeIndex.pentagonsToRegernerate.push( shape.shapeIndex.pentagons[ 0 ] );
				shape.shapeIndex.pentagonsToRegernerate.push( shape.shapeIndex.pentagons[ 1 ] );
				shape.shapeIndex.pentagonsToRegernerate.push( shape.shapeIndex.pentagons[ 2 ] );

				shape.shapeIndex.pentagons.splice( 0 , 3 );

				shape.shapeIndex.regenerate = true;

			} else if( shape.shapes.triangles > 0 ){

				// reduce a side from this shape
				shape.shapes.triangles -=3;
				shape.shapes.total -=3;

				var face = shape.geometry.faces[ shape.shapeIndex.triangles[ shape.shapeIndex.triangleIndex ] ];

				// reset the vertices draw order
				face.a = 0; 
				face.b = 0;
				face.c = 0;

				//face.normal.set( 0 , 0 , 0 );
				shape.geometry.elementsNeedUpdate = true;

				// increment the triangleIndex
				shape.shapeIndex.triangleIndex ++;

			} else if( shape.shapes.squares > 0 ){

				// take a hit off of squares
				shape.shapes.squares -=3;
				shape.shapes.total -=3;

				// check if the square has been shot out.
				// the square takes 10 tetra shots to destroy
				if( shape.shapes.squares % 30 == 0 ){

					var faceOne = shape.geometry.faces[ shape.shapeIndex.squares[ shape.shapeIndex.squareIndex ] ];
					var faceTwo = shape.geometry.faces[ shape.shapeIndex.squares[ shape.shapeIndex.squareIndex + 1 ] ];

					// reset the vertices draw order
					faceOne.a = 0;	faceTwo.a = 0;
					faceOne.b = 0;	faceTwo.b = 0;
					faceOne.c = 0;	faceTwo.c = 0;

					shape.geometry.elementsNeedUpdate = true;

					// increment the triangleIndex
					shape.shapeIndex.squareIndex +=2;
				}
			}

			// check to see if the shape has been destroyed.
			if( shape.shapes.total == 0 ){

				// destroy shape.
				shape.soundFX.play();
				shape.broken = true;

			}
			
			// animation cycle is finished.
			Tower.details.animating = false;
			Tower.soundFX = new Audio("sounds/tetraShot.wav");
			break;
	}
}

function animateCube( Tower , shape ){

	switch( Tower.details.rest ){
		case 1:
			// we'll get the position of the turret and the direction it's pointing in. The position we're looking for is the real
			// world position, not the local one to the tower.
			Tower.updateMatrixWorld();

			// both World variables are the world positions of the two vertices.  The vertices are taken from local space,
			// which is their relative position within the model, then the models scale and position is taken into consideration
			// and the position is updated accordingly.
			
			var barrelStartWorld = Tower.geometry.vertices[ 43 ].clone();
			Tower.barrelEndWorld = Tower.geometry.vertices[ 42 ].clone();
			
			// both local variables are the positions of the vertices relative to the model, not to the world.
			barrelStartLocal = Tower.geometry.vertices[ 43 ].clone();
			barrelEndLocal = Tower.geometry.vertices[ 42 ].clone();

			// transform the vertice from local to world co-ordinates.
			barrelStartWorld.applyMatrix4( Tower.matrixWorld );
			Tower.barrelEndWorld.applyMatrix4( Tower.matrixWorld );

			// we're going to create two vectors reprenting the directions from one end of the barrel to the other
			// to animate the recoil of the barrel we need to know the offset of the vertices within local co-ordinates, not world.
			Tower.barrelOffset = barrelEndLocal.clone().sub( barrelStartLocal ).normalize();
			// to animate the 3D fire burst we need to know  the location of the barrel's end and the direction the barrel is facing
			// in world co-ordinates.
			Tower.barrelOffsetDirection = Tower.barrelEndWorld.clone().sub( barrelStartWorld ).normalize();

			// since offset has been normalized it's distance is effectively 1, so we'll dramatise the distance of the recoil by 
			// multiplying by 1.5.  Now the barrelOffset will be a distance of 1.5
			Tower.barrelOffset.multiplyScalar( 1.5 );

			// lookAtTarget will be the lookAt position for the 3D fire burst animations
			Tower.lookAtTarget = Tower.barrelEndWorld.clone().add( Tower.barrelOffsetDirection );


			Tower.barrelCorners = [];
			Tower.body = [];

			// 8 vertices for tip of barrels within local space
			for( var i=30; i<38; i++ ){
				Tower.barrelCorners[ i - 30 ] = Tower.geometry.vertices[ i ]; 
			}
			
			// 10 vertices for main body of cannon
			for( var i=16; i<26; i++ ){
				Tower.body[ i - 16 ] = Tower.geometry.vertices[ i ]; 
			}

			// We'll update the position of the first frame of the animation to be equal to the end of the barrel in world space.
			Tower.details.animation[ 0 ].position.set( Tower.barrelEndWorld.x , Tower.barrelEndWorld.y , Tower.barrelEndWorld.z );
			Tower.details.animation[ 0 ].lookAt( Tower.lookAtTarget.clone() );
			Tower.details.animation[ 0 ].rotateZ( Math.random() * Math.PI );
			Tower.details.animation[ 0 ].visible = true;

			Tower.soundFX.play();

			break;
		case 2:

			// unload first frame of fire burst animation.
			// load second frame of fire burst animation.
			
			Tower.details.animation[ 0 ].visible = false;
			Tower.details.animation[ 1 ].position.set( Tower.barrelEndWorld.x , Tower.barrelEndWorld.y , Tower.barrelEndWorld.z );
			Tower.details.animation[ 1 ].lookAt( Tower.lookAtTarget.clone() );
			Tower.details.animation[ 1 ].rotateZ( Math.random() * Math.PI );
			Tower.details.animation[ 1 ].visible = true;

			// begin recoil of barrel in local space.
			for( var i=0; i<8; i++ ){
				Tower.barrelCorners[ i ].set( Tower.barrelCorners[ i ].x + ( Tower.barrelOffset.x * 3 ) , Tower.barrelCorners[ i ].y + ( Tower.barrelOffset.y * 3 ) , Tower.barrelCorners[ i ].z + ( Tower.barrelOffset.z * 3 ) ) 
			}
			Tower.geometry.verticesNeedUpdate = true;
			
			break;
		case 3:

			// unload second frame of fire burst animation.
			// load third frame of fire burst animation.			
			
			Tower.details.animation[ 1 ].visible = false;
			Tower.details.animation[ 2 ].position.set( Tower.barrelEndWorld.x , Tower.barrelEndWorld.y , Tower.barrelEndWorld.z );
			Tower.details.animation[ 2 ].lookAt( Tower.lookAtTarget.clone() );
			Tower.details.animation[ 2 ].rotateZ( Math.random() * Math.PI );
			Tower.details.animation[ 2 ].visible = true;

			// continue recoil of barrel in local space.
			for( var i=0; i<8; i++ ){
				Tower.barrelCorners[ i ].set( Tower.barrelCorners[ i ].x + ( Tower.barrelOffset.x * 3 ) , Tower.barrelCorners[ i ].y + ( Tower.barrelOffset.y * 3 ) , Tower.barrelCorners[ i ].z + ( Tower.barrelOffset.z * 3 ) ) 
			}
			// recoil body in local space
			for( var i=0; i<10; i++ ){
				Tower.body[ i ].set( Tower.body[ i ].x + ( Tower.barrelOffset.x * 2 ) , Tower.body[ i ].y + ( Tower.barrelOffset.y * 2 ) , Tower.body[ i ].z + ( Tower.barrelOffset.z * 2 ) );
			}
			Tower.geometry.verticesNeedUpdate = true;
			
			break;

		case 4:

			// unload third frame of fire burst animation.
			Tower.details.animation[ 2 ].visible = false;

			// reset recoil of barrel in local space
			for( var i=0; i<8; i++ ){
				Tower.barrelCorners[ i ].set( Tower.barrelCorners[ i ].x - Tower.barrelOffset.x , Tower.barrelCorners[ i ].y - Tower.barrelOffset.y , Tower.barrelCorners[ i ].z - Tower.barrelOffset.z ); 
			}

			// reset recoil of body in local space
			for( var i=0; i<10; i++ ){
				Tower.body[ i ].set( Tower.body[ i ].x - Tower.barrelOffset.x , Tower.body[ i ].y - Tower.barrelOffset.y , Tower.body[ i ].z - Tower.barrelOffset.z );
			}

			Tower.geometry.verticesNeedUpdate = true;
			break;

		case 5:

			// reset recoil of barrel in local space
			for( var i=0; i<8; i++ ){
				Tower.barrelCorners[ i ].set( Tower.barrelCorners[ i ].x - Tower.barrelOffset.x , Tower.barrelCorners[ i ].y - Tower.barrelOffset.y , Tower.barrelCorners[ i ].z - Tower.barrelOffset.z ); 
			}

			// reset recoil of body in local space
			for( var i=0; i<10; i++ ){
				Tower.body[ i ].set( Tower.body[ i ].x - Tower.barrelOffset.x , Tower.body[ i ].y - Tower.barrelOffset.y , Tower.body[ i ].z - Tower.barrelOffset.z );
			}

			Tower.geometry.verticesNeedUpdate = true;
			break;

		case 6:

			// reset recoil of barrel in local space
			for( var i=0; i<8; i++ ){
				Tower.barrelCorners[ i ].set( Tower.barrelCorners[ i ].x - Tower.barrelOffset.x , Tower.barrelCorners[ i ].y - Tower.barrelOffset.y , Tower.barrelCorners[ i ].z - Tower.barrelOffset.z ); 
			}
			Tower.geometry.verticesNeedUpdate = true;

			// in this frame we will process the shape's hit
			// in order we'll check for pentagons, triangles and then lastly squares.
			// this order is for the tetra-tosser.
			if( shape.shapes.pentagons > 0 ){

				shape.shapes.pentagons -=3;
				shape.shapes.total -=3;

				var faceOne = shape.geometry.faces[ shape.shapeIndex.pentagons[ 0 ] ];
				var faceTwo = shape.geometry.faces[ shape.shapeIndex.pentagons[ 1 ] ];
				var faceThree = shape.geometry.faces[ shape.shapeIndex.pentagons[ 2 ] ];

				// first we'll preserve the original draw oder
				faceOne.oldA = faceOne.a;	faceTwo.oldA = faceTwo.a;	faceThree.oldA = faceThree.a;
				faceOne.oldB = faceOne.b;	faceTwo.oldB = faceTwo.b;	faceThree.oldB = faceThree.b;
				faceOne.oldC = faceOne.c;	faceTwo.oldC = faceTwo.c;	faceThree.oldC = faceThree.c;

				// reset the vertices draw order
				faceOne.a = 0;	faceTwo.a = 0;	faceThree.a = 0;
				faceOne.b = 0;	faceTwo.b = 0;	faceThree.b = 0;
				faceOne.c = 0;	faceTwo.c = 0;	faceThree.c = 0;

				shape.geometry.elementsNeedUpdate = true;

				// remove these three faces from the pentagons array and add them to
				// the pentagonsToRegernerate array
				shape.shapeIndex.pentagonsToRegernerate.push( shape.shapeIndex.pentagons[ 0 ] );
				shape.shapeIndex.pentagonsToRegernerate.push( shape.shapeIndex.pentagons[ 1 ] );
				shape.shapeIndex.pentagonsToRegernerate.push( shape.shapeIndex.pentagons[ 2 ] );

				shape.shapeIndex.pentagons.splice( 0 , 3 );

				shape.shapeIndex.regenerate = true;

			} else if( shape.shapes.squares > 0 ){

				// since the cube shot will destroy an entire square face, we'll check for the remainder
				//  that way if a tetra gun has been hitting the square it will only completely destroy the one sqare, not
				// damage two squares.
				var remainder = shape.shapes.squares % 30;
				if( remainder == 0 ){
					remainder = 30;
				}

				// take a hit off of squares
				shape.shapes.squares -=remainder;
				shape.shapes.total -=remainder;

				// check if the square has been shot out.
				// the square takes 10 tetra shots to destroy
				if( shape.shapes.squares % 30 == 0 ){

					var faceOne = shape.geometry.faces[ shape.shapeIndex.squares[ shape.shapeIndex.squareIndex ] ];
					var faceTwo = shape.geometry.faces[ shape.shapeIndex.squares[ shape.shapeIndex.squareIndex + 1 ] ];

					// reset the vertices draw order
					faceOne.a = 0;	faceTwo.a = 0;
					faceOne.b = 0;	faceTwo.b = 0;
					faceOne.c = 0;	faceTwo.c = 0;

					shape.geometry.elementsNeedUpdate = true;

					// increment the triangleIndex
					shape.shapeIndex.squareIndex +=2;
				}
			} else if( shape.shapes.triangles > 0 ){

				// reduce a side from this shape
				shape.shapes.triangles -=3;
				shape.shapes.total -=3;

				var face = shape.geometry.faces[ shape.shapeIndex.triangles[ shape.shapeIndex.triangleIndex ] ];

				// reset the vertices draw order
				face.a = 0; 
				face.b = 0;
				face.c = 0;

				//face.normal.set( 0 , 0 , 0 );
				shape.geometry.elementsNeedUpdate = true;

				// increment the triangleIndex
				shape.shapeIndex.triangleIndex ++;

			}

			// check to see if the shape has been destroyed.
			if( shape.shapes.total == 0 ){

				// destroy shape.
				shape.soundFX.play();
				shape.broken = true;

			}
			
			break;

		case 7:

			// reset recoil of barrel in local space
			for( var i=0; i<8; i++ ){
				Tower.barrelCorners[ i ].set( Tower.barrelCorners[ i ].x - Tower.barrelOffset.x , Tower.barrelCorners[ i ].y - Tower.barrelOffset.y , Tower.barrelCorners[ i ].z - Tower.barrelOffset.z ); 
			}
			Tower.geometry.verticesNeedUpdate = true;
			break;

		case 8:

			// reset recoil of barrel in local space
			for( var i=0; i<8; i++ ){
				Tower.barrelCorners[ i ].set( Tower.barrelCorners[ i ].x - Tower.barrelOffset.x , Tower.barrelCorners[ i ].y - Tower.barrelOffset.y , Tower.barrelCorners[ i ].z - Tower.barrelOffset.z ); 
			}
			Tower.geometry.verticesNeedUpdate = true;
			break;

		case 9:

			// reset recoil of barrel in local space
			for( var i=0; i<8; i++ ){
				Tower.barrelCorners[ i ].set( Tower.barrelCorners[ i ].x - Tower.barrelOffset.x , Tower.barrelCorners[ i ].y - Tower.barrelOffset.y , Tower.barrelCorners[ i ].z - Tower.barrelOffset.z ); 
			}
			Tower.geometry.verticesNeedUpdate = true;

			// animation cycle is finished.
			Tower.details.animating = false;
			Tower.soundFX = new Audio("sounds/cubeCannon.wav");
			break;

	}

}