var mouseControls = mouseControls || {};

//create objects used to raycaste with a mouse click;
mouseVector2 = new THREE.Vector2();
mouseRay = new THREE.Raycaster();

///////////////////////////////////////////////////////////////////
// WALL PLACEMENT
///////////////////////////////////////////////////////////////////

mouseControls.checkMouseDownForWallPlacement = function( event ){

	// we'll assume the the vertices will be stored in cells.previousSelection

	// first will see if cells.previousSelection is populated with any vertices
	if( cells.previousSelection.length > 0 ){

		// we're in luck
		// next we'll check if the wall already exists at this location
		if( walls.lookUpArray[ cells.previousSelection[0] ] ){

			// the wall already exists.  check if the user wants to delete the wall
			if( event.buttons == 1 ){

				// no further action required

			} else {

				// the user has right clicked on the wall, so we're going to remove it 
				// and add its vertice location back to the availability pool

				var indexToClear = walls.bufferLink[ cells.previousSelection[0] ];

				// reset vertices to 0,0,0
				walls.geometry.vertices[ indexToClear ].set( 0 , 0 , 0 );
				walls.geometry.vertices[ indexToClear+1 ].set( 0 , 0 , 0 );
				walls.geometry.vertices[ indexToClear+2 ].set( 0 , 0 , 0 );
				walls.geometry.vertices[ indexToClear+3 ].set( 0 , 0 , 0 );

				// Now we update the travelTo array to allow movement.
				mouseFunctions.updateCell( cells, true , "travelTo" );

				// add index position to the availability pool
				walls.availableIndexes.push( indexToClear );

				// set lookUpArray to false
				walls.lookUpArray[ cells.previousSelection[0] ] = false;

				// reset walls.infoArray
				walls.infoArray[ cells.previousSelection[0] ] = [];

				// now that we've set the location of the vertices we'll need to update the geometry.
				// the faces will automatically shrink into place as each set of 2 faces is connected to
				// 4 vertices.
				walls.geometry.computeFaceNormals();
				walls.geometry.verticesNeedUpdate = true;

			}

		} else {

			// now we'll see if the walls bufferIndex has any more room
			if( walls.bufferIndex.vertices == walls.bufferIndex.maxVertices ){

				// the maximum number of walls has ben created, check to see if there
				// is any room in the availableIndexes array
				if( walls.availableIndexes.length == 0 ){

					// maximum walls created.
					// no further action required

				} else {

					// create a wall using an index that has been freed up.
					// but first we'll check to see if creating this wall blocks
					// the passage to an exit.
					if( mouseFunctions.checkForClearPassage( cells ) ){

						mouseFunctions.createWall( walls.availableIndexes[0] );

						// add an entry into buffer Link
						walls.bufferLink[ cells.previousSelection[0] ] = walls.availableIndexes[0];

						// remove available Index
						walls.availableIndexes.splice( 0 , 1 );
					} 					

				}

			} else {

				// create a wall using a new index
				// but first we'll check to see if creating this wall blocks
				// the passage to an exit.
				if( mouseFunctions.checkForClearPassage( cells ) ){

					mouseFunctions.createWall( walls.bufferIndex.vertices );

					// add an entry into buffer Link
					walls.bufferLink[ cells.previousSelection[0] ] = walls.bufferIndex.vertices;

					// lastly, update the buffer index
					walls.bufferIndex.vertices +=4;
				} 
			}
		}
	}
}

mouseControls.checkMouseMoveForWallPlacement = function( event ){

	if( camera.renderSizeToggle ){
		mouseVector2.x = ( event.clientX / ( window.innerWidth / 2 ) ) - 1;
		mouseVector2.y = - ( event.clientY / ( window.innerHeight / 2 ) ) + 1;
	} else {
		mouseVector2.x = ( event.clientX / ( 8+( 1024 / 2 )) ) - 1;
		mouseVector2.y = - ( event.clientY / ( 8+( 768 / 2 )) ) + 1;
	}

	mouseRay.setFromCamera( mouseVector2 , camera );

	var intersect = mouseRay.intersectObject( gridTargetMesh );

	if( intersect.length > 0 ){

		// reset colors of all previous lines that were selected
		var baseColor; 
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			baseColor = grid.geometry.colors[ cells.previousSelection[i] ].base;
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( baseColor.r , baseColor.g , baseColor.b );
		}

		// reset previous selection of lines.
		cells.previousSelection = [];

		// change the color of the line we've selected
		var face = intersect[ 0 ].face;
		var vectorList = cells.list[ face.index ].lineVertices;

		// highlight the closest line if within a certain distance.
		// this way it will seem a little more intuitive when selecting a line.
		var closestDistance = 10;
		var temporaryDistance = 0;
		var verticesIndex = -1;
		var allowedDistance = 0.1065; // This number seems to have the right feel to it.

		// we're going to calculate the distance from the intersected point 
		// to each set of vertices used to draw the grid lines, only if it's short enough will
		// we allow a selection.  
		for( var i=0; i<vectorList.length; i+=2 ){
			temporaryDistance = grid.geometry.vertices[ vectorList[i] ].distanceTo( intersect[0].point ) + 
								grid.geometry.vertices[ vectorList[ i+1 ] ].distanceTo( intersect[0].point );
			if( temporaryDistance < closestDistance && temporaryDistance <= allowedDistance ){
				closestDistance = temporaryDistance;
				verticesIndex = vectorList[i];

				// we're going to record the side that we're going to create a wall at
				// this will come in handy when we update cells.list[].travelTo later.
				// the cell to the left = 0, up = 1, right = 2, and down = 3.
				cells.info = [ { faceIndex: face.index , side: i*0.5 } ];
			}
		}
		
		// check to see if a line has been found.
		// we'll also check to see if the the faceIndex is a starting location or ending location
		// in that case we won't allow a wall to be built.

		if( verticesIndex > -1 && ( !cells.list[ face.index ].start && !cells.list[ face.index ].goal ) ){

			// now we'll highlight the line closest to the intersection point.
			grid.geometry.colors[ verticesIndex ].setRGB( 1 , 1 , 0 );
			grid.geometry.colors[ verticesIndex + 1 ].setRGB( 1 , 1 , 0 );			
			cells.previousSelection.push( verticesIndex );
			cells.previousSelection.push( verticesIndex + 1 );	
		} else {
			// rest cells.side.
			cells.info = [];
		}

	} else {

		// reset colors of all previous lines that were selected
		var baseColor; 
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			baseColor = grid.geometry.colors[ cells.previousSelection[i] ].base;
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( baseColor.r , baseColor.g , baseColor.b );
		}

		// reset previous selection of lines.
		cells.previousSelection = [];

	}

	grid.geometry.colorsNeedUpdate = true;

	if( !mouseFunctions.checkForClearPassage( cells ) ){

		// BUG FOUND
		// the first bug has been found, and for the scope of this project we're going to eliminate it the easy way.
		// To eliminate the bug we're going to test the same wall location but from the opposite faceIndex
		// seems that sometimes if tested from one index it returns false, but from the connecting face's index returns true.
		// I assume it has to do with the indexed order of the surroundingCells array.
		var sideReciprical = [ 2 , 3 , 0 , 1 ];
		
		cells.info = [ { faceIndex: cells.list[ face.index ].surroundingCells[ cells.info[ 0 ].side * 2 ] , side: sideReciprical[ cells.info[ 0 ].side ] } ];
		
		if( !mouseFunctions.checkForClearPassage( cells ) ){

			// turn the color of the line purple to indicate that the wall can't be built
			for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
				grid.geometry.colors[ cells.previousSelection[i] ].setRGB( 1 , 0 , 0 );
			}
		} 
	}
}

///////////////////////////////////////////////////////////////////
// TOWER PLACEMENT
///////////////////////////////////////////////////////////////////

mouseControls.checkMouseDownForTowerPlacement = function( event ){

	if( camera.renderSizeToggle ){
		mouseVector2.x = ( event.clientX / ( window.innerWidth / 2 ) ) - 1;
		mouseVector2.y = - ( event.clientY / ( window.innerHeight / 2 ) ) + 1;
	} else {
		mouseVector2.x = ( event.clientX / ( 8+( 1024 / 2 )) ) - 1;
		mouseVector2.y = - ( event.clientY / ( 8+( 768 / 2 )) ) + 1;
	}

	mouseRay.setFromCamera( mouseVector2 , camera );

	var intersect = mouseRay.intersectObject( gridTargetMesh );

	// check to see if the user wants to delete a tower that may exist at this location
	if( event.buttons != 1 ){

		// the user is wanting to delete a tower at this location.
		if( intersect.length > 0 ){

			// we'll check to see if a tower exists at this location
			var face = intersect[ 0 ].face;

			if( cells.list[ face.index ].towerID > -1 ){

				// a tower exists at this location, now we will remove it.
				var vectorList = [];
				var faceIndex = [];
				var color = { r:0 , g:1, b:1 };

				// we're going to record the face index position which will be needed update the passage arrays.
				faceIndex[0] = cells.list[ face.index ].surroundingCells[ 0 ];
				faceIndex[1] = cells.list[ face.index ].surroundingCells[ 1 ];
				faceIndex[2] = cells.list[ face.index ].surroundingCells[ 2 ];
				faceIndex[3] = face.index;

				// we're going to collect the face indexes of four squares 2x2
				vectorList[0] = cells.list[ cells.list[ face.index ].surroundingCells[ 0 ] ].lineVertices;
				vectorList[1] = cells.list[ cells.list[ face.index ].surroundingCells[ 1 ] ].lineVertices;
				vectorList[2] = cells.list[ cells.list[ face.index ].surroundingCells[ 2 ] ].lineVertices;
				vectorList[3] = cells.list[ face.index ].lineVertices;

				// we're going to collect the grid lines under the tower
				var underList =[];
				var extraInfo =[];

				// we're going to clear the sides array so that we can add each side to the list to be tested later on
				// for clear passage to the goal.
				cells.info = [];

				// then we're going to only draw their corners.
				for( var i=0, a=0, b=6; i<4; i++ ){

					// we're going to see if i = 0 or 2
					// if so we're going to get the lineVertices so that we can change the color later
					if( i == 0 ){
						underList.push( vectorList[ i ][ 2 ] );
						underList.push( vectorList[ i ][ 3 ] );
						underList.push( vectorList[ i ][ 4 ] );
						underList.push( vectorList[ i ][ 5 ] );
						extraInfo.push( { faceIndex: faceIndex[i] , side:1 } );
						extraInfo.push( { faceIndex: faceIndex[i] , side:2 } );
					} else if( i == 2 ){
						underList.push( vectorList[ i ][ 0 ] );
						underList.push( vectorList[ i ][ 1 ] );
						underList.push( vectorList[ i ][ 6 ] );
						underList.push( vectorList[ i ][ 7 ] );
						extraInfo.push( { faceIndex: faceIndex[i] , side:0 } );
						extraInfo.push( { faceIndex: faceIndex[i] , side:3 } );
					}

					if( vectorList[ i ][ a ] != undefined ){
						grid.geometry.colors[ vectorList[ i ][ a ] ].setRGB( color.r , color.g , color.b );
						grid.geometry.colors[ vectorList[ i ][ a+1 ] ].setRGB( color.r , color.g , color.b );
						grid.geometry.colors[ vectorList[ i ][ b ] ].setRGB( color.r , color.g , color.b );
						grid.geometry.colors[ vectorList[ i ][ b+1 ] ].setRGB( color.r , color.g , color.b );

						// add which sides need to be updated
						cells.info.push( { faceIndex: faceIndex[i] , side:a*0.5 } );
						cells.info.push( { faceIndex: faceIndex[i] , side:b*0.5 } );
						a+=2;	
						b+=2;
						if( b > 6 ){ b = 0; }
					}
				}

				mouseFunctions.removeTower( underList , extraInfo , face.index );

			} else {

				// a tower does not exist at this location, no further action is required.

			}
		}

	} else {

		// check to see if we intersected the gridTargetMesh
		if( intersect.length > 0 ){

			// the user is wanting to create a tower at this location.

			// reset colors of all previous cells that were selected
			var baseColor; 
			for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
				baseColor = grid.geometry.colors[ cells.previousSelection[i] ].base;
				grid.geometry.colors[ cells.previousSelection[i] ].setRGB( baseColor.r , baseColor.g , baseColor.b );
			}

			// reset previous selection of cells.
			cells.previousSelection = [];

			// change the color of the square we've selected
			var face = intersect[ 0 ].face;
			
			// first we're going to see if the faceIndex is a starting location.
			// if it is we're going to prevent prevent this function from firing.
			if( ( !cells.list[ face.index ].start && !cells.list[ face.index ].goal ) && !cells.list[ cells.list[ face.index ].surroundingCells[ 0 ] ].goal ){ 

				var vectorList = [];
				var faceIndex = [];
				
				// first we'll check to see if this spot can have a tower.  But we're not yet checking if
				// placing this tower will block passage, only checking if there Is there passage allowed to the 
				// are any other obsticles in the way, such as a wall or another tower.
				var canBuildTowerHere = false;
				// when we draw the temporary square we'll use the following color scheme
				var color = { r:1, g:0, b:0 };
				if( cells.list[ face.index ].travelTo[ 0 ] && cells.list[ face.index ].travelTo[ 1 ] && cells.list[ face.index ].travelTo[ 2 ] ){
					canBuildTowerHere = true;
					color = { r:0, g:1, b:1 };
				}

				// we're going to record the face index position which will be needed when we check for clear passage.
				faceIndex[0] = cells.list[ face.index ].surroundingCells[ 0 ];
				faceIndex[1] = cells.list[ face.index ].surroundingCells[ 1 ];
				faceIndex[2] = cells.list[ face.index ].surroundingCells[ 2 ];
				faceIndex[3] = face.index;

				// we're going to collect the face indexes of four squares 2x2
				vectorList[0] = cells.list[ cells.list[ face.index ].surroundingCells[ 0 ] ].lineVertices;
				vectorList[1] = cells.list[ cells.list[ face.index ].surroundingCells[ 1 ] ].lineVertices;
				vectorList[2] = cells.list[ cells.list[ face.index ].surroundingCells[ 2 ] ].lineVertices;
				vectorList[3] = cells.list[ face.index ].lineVertices;

				// we're going to collect the grid lines under any tower that's successfully built.
				var underList =[];
				var extraInfo =[];


				// we're going to clear the sides array so that we can add each side to the list to be tested later on
				// for clear passage to the goal.
				cells.info = [];

				// then we're going to only draw their corners.
				for( var i=0, a=0, b=6; i<4; i++ ){

					// we're going to see if i = 0 or 2
					// if so we're going to get the lineVertices so that we can change the color later
					if( i == 0 ){
						underList.push( vectorList[ i ][ 2 ] );
						underList.push( vectorList[ i ][ 3 ] );
						underList.push( vectorList[ i ][ 4 ] );
						underList.push( vectorList[ i ][ 5 ] );
						extraInfo.push( { faceIndex: faceIndex[i] , side:1 } );
						extraInfo.push( { faceIndex: faceIndex[i] , side:2 } );
					} else if( i == 2 ){
						underList.push( vectorList[ i ][ 0 ] );
						underList.push( vectorList[ i ][ 1 ] );
						underList.push( vectorList[ i ][ 6 ] );
						underList.push( vectorList[ i ][ 7 ] );
						extraInfo.push( { faceIndex: faceIndex[i] , side:0 } );
						extraInfo.push( { faceIndex: faceIndex[i] , side:3 } );
					}

					if( vectorList[ i ][ a ] != undefined ){
						grid.geometry.colors[ vectorList[ i ][ a ] ].setRGB( color.r , color.g , color.b );
						cells.previousSelection.push( vectorList[ i ][ a ] );
						grid.geometry.colors[ vectorList[ i ][ a+1 ] ].setRGB( color.r , color.g , color.b );
						cells.previousSelection.push( vectorList[ i ][ a+1 ] );
						grid.geometry.colors[ vectorList[ i ][ b ] ].setRGB( color.r , color.g , color.b );
						cells.previousSelection.push( vectorList[ i ][ b ] );
						grid.geometry.colors[ vectorList[ i ][ b+1 ] ].setRGB( color.r , color.g , color.b );
						cells.previousSelection.push( vectorList[ i ][ b+1 ] );

						// add which sides need to be tested, and for which cell index.
						cells.info.push( { faceIndex: faceIndex[i] , side:a*0.5 } );
						cells.info.push( { faceIndex: faceIndex[i] , side:b*0.5 } );
						a+=2;	
						b+=2;
						if( b > 6 ){ b = 0; }
					}
				}
			}

		} else {

			// if we click outside the play area we'll reset the colors of the last selected square.
			var baseColor; 
			for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
				baseColor = grid.geometry.colors[ cells.previousSelection[i] ].base;
				grid.geometry.colors[ cells.previousSelection[i] ].setRGB( baseColor.r , baseColor.g , baseColor.b );
			}

			// reset previous selection of cells.
			cells.previousSelection = [];

		}

		if( !mouseFunctions.checkForClearPassage( cells ) || !canBuildTowerHere ){

			// turn the color of the line purple to indicate that the tower can't be built
			for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
				grid.geometry.colors[ cells.previousSelection[i] ].setRGB( 1 , 0 , 0 );
			}

			cells.previousSelection = [];

		} else {

			// we'll build the tower at this location if previousSelection is populated.
			if( cells.previousSelection.length > 0 ){			

				// next we'll check to see if there are any towers of this type that can be built.
				if( gameParameters.tower[ gameParameters.tower.previousSelection ].built < gameParameters.tower[ gameParameters.tower.previousSelection ].allowed ){

					// we'll increment the built towers
					gameParameters.tower[ gameParameters.tower.previousSelection ].built ++;

					// we're going to change the color to gree to indicate the tower has been built.
					for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
						grid.geometry.colors[ cells.previousSelection[i] ].setRGB( 1 , 0 , 0 );
					}

					mouseFunctions.createTower( underList , extraInfo , face.index );		

				}
			}
		}

		grid.geometry.colorsNeedUpdate = true;

	}
}

mouseControls.checkMouseMoveForTowerPlacement = function( event ){


	if( camera.renderSizeToggle ){
		mouseVector2.x = ( event.clientX / ( window.innerWidth / 2 ) ) - 1;
		mouseVector2.y = - ( event.clientY / ( window.innerHeight / 2 ) ) + 1;
	} else {
		mouseVector2.x = ( event.clientX / ( 8+( 1024 / 2 )) ) - 1;
		mouseVector2.y = - ( event.clientY / ( 8+( 768 / 2 )) ) + 1;
	}

	mouseRay.setFromCamera( mouseVector2 , camera );

	var intersect = mouseRay.intersectObject( gridTargetMesh );

	// check to see if we intersected the gridTargetMesh
	if( intersect.length > 0 ){

		// reset colors of all previous cells that were selected
		var baseColor; 
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			baseColor = grid.geometry.colors[ cells.previousSelection[i] ].base;
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( baseColor.r , baseColor.g , baseColor.b );
		}

		// reset previous selection of cells.
		cells.previousSelection = [];

		// change the color of the square we've selected
		var face = intersect[ 0 ].face;
		
		// first we're going to see if the faceIndex is a starting location or goal.
		// if it is we're going to prevent prevent this function from firing.
		if( ( !cells.list[ face.index ].start && !cells.list[ face.index ].goal ) && !cells.list[ cells.list[ face.index ].surroundingCells[ 0 ] ].goal ){ 

			var vectorList = [];
			var faceIndex = [];
			
			// first we'll check to see if this spot can have a tower.  But we're not yet checking if
			// placing this tower will block passage, only checking if there Is there passage allowed to the 
			// are any other obsticles in the way, such as a wall or another tower.
			var canBuildTowerHere = false;
			// when we draw the temporary square we'll use the following color scheme
			var color = { r:1, g:0, b:0 };
			if( cells.list[ face.index ].travelTo[ 0 ] && cells.list[ face.index ].travelTo[ 1 ] && cells.list[ face.index ].travelTo[ 2 ] ){
				canBuildTowerHere = true;
				color = { r:0, g:1, b:1 };
			}

			// we're going to record the face index position which will be needed when we check for clear passage.
			faceIndex[0] = cells.list[ face.index ].surroundingCells[ 0 ];
			faceIndex[1] = cells.list[ face.index ].surroundingCells[ 1 ];
			faceIndex[2] = cells.list[ face.index ].surroundingCells[ 2 ];
			faceIndex[3] = face.index;

			// we're going to collect the face indexes of four squares 2x2
			vectorList[0] = cells.list[ cells.list[ face.index ].surroundingCells[ 0 ] ].lineVertices;
			vectorList[1] = cells.list[ cells.list[ face.index ].surroundingCells[ 1 ] ].lineVertices;
			vectorList[2] = cells.list[ cells.list[ face.index ].surroundingCells[ 2 ] ].lineVertices;
			vectorList[3] = cells.list[ face.index ].lineVertices;


			// we're going to clear the sides array so that we can add each side to the list to be tested later on
			// for clear passage to the goal.
			cells.info = [];

			// then we're going to only draw their corners.
			for( var i=0, a=0, b=6; i<4; i++ ){
				if( vectorList[ i ][ a ] != undefined ){
					grid.geometry.colors[ vectorList[ i ][ a ] ].setRGB( color.r , color.g , color.b );
					cells.previousSelection.push( vectorList[ i ][ a ] );
					grid.geometry.colors[ vectorList[ i ][ a+1 ] ].setRGB( color.r , color.g , color.b );
					cells.previousSelection.push( vectorList[ i ][ a+1 ] );
					grid.geometry.colors[ vectorList[ i ][ b ] ].setRGB( color.r , color.g , color.b );
					cells.previousSelection.push( vectorList[ i ][ b ] );
					grid.geometry.colors[ vectorList[ i ][ b+1 ] ].setRGB( color.r , color.g , color.b );
					cells.previousSelection.push( vectorList[ i ][ b+1 ] );

					// add which sides need to be tested, and for which cell index.
					cells.info.push( { faceIndex: faceIndex[i] , side:a*0.5 } );
					cells.info.push( { faceIndex: faceIndex[i] , side:b*0.5 } );
					a+=2;	
					b+=2;
					if( b > 6 ){ b = 0; }
				}
			}
		}

	} else {

		// if we click outside the play area we'll reset the colors of the last selected square.
		var baseColor; 
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			baseColor = grid.geometry.colors[ cells.previousSelection[i] ].base;
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( baseColor.r , baseColor.g , baseColor.b );
		}

		// reset previous selection of cells.
		cells.previousSelection = [];

	}

	grid.geometry.colorsNeedUpdate = true;

	if( !mouseFunctions.checkForClearPassage( cells ) ){

		// turn the color of the line purple to indicate that the tower can't be built
		for( var i=0, ii=cells.previousSelection.length; i<ii; i++ ){
			grid.geometry.colors[ cells.previousSelection[i] ].setRGB( 1 , 0 , 0 );
		}
	}
}