var mouseFunctions = mouseFunctions || {};

mouseFunctions.checkForClearPassage = function( cells ){

	var aCell;

	// set all currentIteration arrays to their travelTo state.
	for( var i=0, ii=cells.list.length; i<ii; i++ ){
		aCell = cells.list[ i ];
		for( var j=0; j<8; j++ ){
			aCell.currentIteration[ j ] = aCell.travelTo[ j ];
		}
	}

	// first we'll pretend that the wall has been added.
	// then we'll test to see if the path is clear
	updateCell( cells , false , "currentIteration" );

	// path will be an array that we add to.  At each iteration of our Flood Fill pathfinding algorithm
	// the path will copy its self for each subsequent iteration and add a unique bit to each new construct.  
	// Each Path item will contain an array of all spaces it has traveled to and begin with the overall distance.
	// For now the path will start at the last square.
	// the first entry in the path array will be an object indicating whether or not an end goal has been found.
	// and what the indexed position is for the shortest path and the length of the shortest path.
	var path = [ { goal: false , index: -1, length: 10000 } , [ 0 , ( cells.list.length - 1 ) ] ];

	iterateThroughAllDirections( cells , path );

	return path[0].goal;
}

mouseFunctions.createWall = function( index ){

	// the wall does not exist and there is room to build.  
	// we'll create one at this location.
	walls.lookUpArray[ cells.previousSelection[0] ] = true;

	// we'll add an entry into the infoArray
	walls.infoArray[ cells.previousSelection[0] ] = { faceIndex: cells.info[0].faceIndex , side: cells.info[0].side };

	// assign two temporary placeholder variables
	var v1 = grid.geometry.vertices[ cells.previousSelection[0] ];
	var v2 = grid.geometry.vertices[ cells.previousSelection[1] ];

	walls.geometry.vertices[ index ].set( v1.x , v1.y , v1.z );
	walls.geometry.vertices[ index+1 ].set( v1.x , v1.y+0.02 , v1.z );

	walls.geometry.vertices[ index+2 ].set( v2.x , v2.y , v2.z );
	walls.geometry.vertices[ index+3 ].set( v2.x , v2.y+0.02 , v2.z );

	// Now we update the travelTo array to restrict movement.
	updateCell( cells , false , "travelTo" );

	// now that we've set the location of the vertices we'll need to update the geometry.
	// the faces will automatically stretch into place as each set of 2 faces is connected to
	// 4 vertices.
	walls.geometry.verticesNeedUpdate = true;
	walls.geometry.facesNeedUpdate = true;
	walls.geometry.normalsNeedUpdate = true;
	walls.geometry.computeFaceNormals();
	walls.geometry.computeBoundingSphere();
	walls.updateMatrixWorld();
}

mouseFunctions.createTower = function( underList , extraInfo , faceIndex ){

	// we're going to update the towerID of the cell.  that way later on 
	// if we want to remove the tower by right clicking over top of the tower
	// we will know which tower occupies this cell and surrounding cells.
	cells.list[ faceIndex ].towerID = towers.length;

	// we're going to create a line and capture it's center point for placing the tower.
	var line3 = new THREE.Line3( 
		grid.geometry.vertices[ cells.previousSelection[ 0 ] ].clone(),
		grid.geometry.vertices[ cells.previousSelection[ 9 ] ].clone() 
	);

	var centerVector = line3.at( 0.5 );

	var basicMaterial = new THREE.MeshLambertMaterial( { vertexColors: THREE.FaceColors } );
	var tower = {
		type: gameParameters.tower.previousSelection,
		base: new THREE.Mesh( models.import( gameParameters.tower.modelBase , 10 ) , basicMaterial ),
		turret : new THREE.Mesh( models.import( gameParameters.tower.modelTurret , 10 ) , basicMaterial ),
		infoArray: [], // this will come in handy when removing towers
		bufferLink:[], // this will come in handy when removing towers
		soundFX: null, // we'll load a sound FX each time the tower fires a shot.
	};
	
	// scale and position tower's base and turret.
	tower.base.scale.set( 0.01 , 0.01 , 0.01 );
	tower.base.position.set( centerVector.x , centerVector.y , centerVector.z );
	tower.base.rotateY( Math.random() * Math.PI );
	tower.base.visible = true;

	tower.turret.scale.set( 0.01 , 0.01 , 0.01 );
	tower.turret.position.set( centerVector.x , centerVector.y , centerVector.z );
	tower.turret.rotateY( Math.random() * Math.PI );
	tower.turret.visible = true;

	// we're going to add the details to this tower's turret.
	tower.turret.details = generateGameContent.towerDetails( gameParameters.tower.previousSelection );
	console.log( tower );

	// next we'll change the colors of the grid lines under the placed tower
	// so that it will look like they've disappeared.
	var baseColor; 
	for( var i=0, ii=underList.length; i<ii; i++ ){
		baseColor = grid.geometry.colors[ underList[ i ] ].base;
		baseColor.r = 0; 
		baseColor.g = 0; 
		baseColor.b = 0;
		grid.geometry.colors[ underList[ i ] ].setRGB( baseColor.r , baseColor.g , baseColor.b );
	}

	var bufferIndexToUse;

	// we're going to create a wall around each tower.
	for( var i=0, ii=cells.previousSelection.length; i<ii; i+=2 ){

		// we'll check which senario we're going to use for retrieving a bufferIndex location.
		if( towerWalls.bufferIndex.vertices == towerWalls.bufferIndex.maxVertices ){
			
			bufferIndexToUse = towerWalls.availableIndexes[0];

			// add an entry into buffer Link
			tower.bufferLink[ cells.previousSelection[i] ] = bufferIndexToUse;

			// remove this index entry from available indexes
			towerWalls.availableIndexes.splice( 0 , 1 );

		} else {

			bufferIndexToUse = towerWalls.bufferIndex.vertices;

			// add an entry into buffer Link
			tower.bufferLink[ cells.previousSelection[i] ] = towerWalls.bufferIndex.vertices;

			// increment bufferIndex.vertices.
			towerWalls.bufferIndex.vertices +=4;

		}

		// assign two temporary placeholder variables
		var v1 = grid.geometry.vertices[ cells.previousSelection[i] ];
		var v2 = grid.geometry.vertices[ cells.previousSelection[i+1] ];

		// create two walls at a corner of the cell
		towerWalls.geometry.vertices[ bufferIndexToUse ].set( v1.x , v1.y , v1.z );
		towerWalls.geometry.vertices[ bufferIndexToUse+1 ].set( v1.x , v1.y+0.02 , v1.z );

		towerWalls.geometry.vertices[ bufferIndexToUse+2 ].set( v2.x , v2.y , v2.z );
		towerWalls.geometry.vertices[ bufferIndexToUse+3 ].set( v2.x , v2.y+0.02 , v2.z );		
		
	}

	// now that we've set the location of the vertices we'll need to update the geometry.
	// the faces will automatically stretch into place as each set of 2 faces is connected to
	// 4 vertices.
	towerWalls.geometry.verticesNeedUpdate = true;
	towerWalls.geometry.facesNeedUpdate = true;
	towerWalls.geometry.normalsNeedUpdate = true;
	towerWalls.geometry.computeFaceNormals();
	towerWalls.geometry.computeBoundingSphere();
	towerWalls.updateMatrixWorld();

	// finally we'll update the surrounding cell travelTo Information.
	// first we'll add underList to previousSelection.  Then when we send it off to 
	// updateCell() it'll update all accordingly.
	for( var i=0, ii=underList.length; i<ii; i++ ){
		cells.previousSelection.push( underList[ i ] );
	}
	// we'll add the faceIndex and side info to cells.info[];
	for( var i=0, ii=extraInfo.length; i<ii; i++ ){
		cells.info.push( extraInfo[ i ] );
	}

	// we're going to add each info entry into the tower's info array
	for( var i=0, ii=cells.info.length; i<ii; i++ ){
		tower.infoArray.push( { faceIndex: cells.info[i].faceIndex , side: cells.info[i].side } );
	}

	updateCell( cells , false , "travelTo" );

	// The next part will help us process an tower deletion if the user changes their mind about
	// where they want the tower, but don't move the mouse at all.
	for( var i=0, ii=underList.length; i<ii; i++ ){
		cells.previousSelection.pop();
	}
	for( var i=0, ii=extraInfo.length; i<ii; i++ ){
		cells.info.pop();
	}

	// add the tower to the towers array and add the base and turret to the main scene.
	towers.push( tower );
	scene.add( towers[ towers.length-1 ].base );
	scene.add( towers[ towers.length-1 ].turret );
}

mouseFunctions.removeTower = function( underList , extraInfo , faceIndex ){

	var towerID = cells.list[ faceIndex ].towerID;

	// first we're going to collapse the walls surrounding the tower
	// we're going to create a wall around each tower.
	var bufferIndexToUse;
	for( var i=0, ii=cells.previousSelection.length; i<ii; i+=2 ){

		// we'll grab the starting index of this position for the bufferIndex
		bufferIndexToUse = towers[ towerID ].bufferLink[ cells.previousSelection[i] ];
		
		towerWalls.geometry.vertices[ bufferIndexToUse ].set( 0 , 0 , 0 );
		towerWalls.geometry.vertices[ bufferIndexToUse+1 ].set( 0 , 0 , 0 );

		towerWalls.geometry.vertices[ bufferIndexToUse+2 ].set( 0 , 0 , 0 );
		towerWalls.geometry.vertices[ bufferIndexToUse+3 ].set( 0 , 0 , 0 );

		// add an entry into available indexes
		towerWalls.availableIndexes.push( bufferIndexToUse );
		
	}

	// now that we've set the location of the vertices we'll need to update the geometry.
	// the faces will automatically shrink into place as each set of 2 faces is connected to
	// 4 vertices.
	towerWalls.geometry.verticesNeedUpdate = true;
	towerWalls.geometry.facesNeedUpdate = true;
	towerWalls.geometry.normalsNeedUpdate = true;
	towerWalls.geometry.computeFaceNormals();
	towerWalls.geometry.computeBoundingSphere();
	towerWalls.updateMatrixWorld();

	// now we'll remove the tower and the turret from the scene
	scene.remove( towers[ towerID ].base );
	scene.remove( towers[ towerID ].turret );

	// we'll subtract from total towers built of this kind
	gameParameters.tower[ towers[ towerID ].type ].built --;

	// now we'll remove the towerID from the cell
	cells.list[ faceIndex ].towerID = -1;

	// we'll cycle through all cells looking for towerID's that are greater than towerID
	// if they are greater than towerID we'll update them by subtracting one.
	for( var i=0, ii=cells.list.length; i<ii; i++ ){
		if( cells.list[ i ].towerID > towerID ){
			cells.list[ i ].towerID --;
		}
	}
	// we'll remove this entry from the towers array
	towers.splice( towerID , 1 );

	// next we'll change the colors of the grid lines under the placed tower
	// so that it will look like they're visible again.
	var baseColor; 
	for( var i=0, ii=underList.length; i<ii; i++ ){
		baseColor = grid.geometry.colors[ underList[ i ] ].base;
		baseColor.r = 0.3; 
		baseColor.g = 0.3; 
		baseColor.b = 0.3;
		grid.geometry.colors[ underList[ i ] ].setRGB( baseColor.r , baseColor.g , baseColor.b );
	}
	grid.geometry.colorsNeedUpdate = true;
	// we'll update the surrounding cell travelTo Information.
	// first we'll add underList to previousSelection.  Then when we send it off to 
	// updateCell() it'll update all accordingly.
	for( var i=0, ii=underList.length; i<ii; i++ ){
		cells.previousSelection.push( underList[ i ] );
	}
	// we'll add the faceIndex and side info to cells.info[];
	for( var i=0, ii=extraInfo.length; i<ii; i++ ){
		cells.info.push( extraInfo[ i ] );
	}

	// we'll need go through walls.lookUpArray to check for any true conditions.
	// if there are any we'll need to remove that grid from the cells.info list.
	// so that any previously built walls will remain active
	// we will also cycle through all towers and do the same thing with their infoArrays;
	
	// cycle through all walls built
	// we will also need to test the recipricals of all faceIndex's. This is because a wall or tower can 
	// be built from two sides of a given grid line.
	var sideReciprical = [ 2 , 3 , 0 , 1 ];
	for( var i=0, ii=walls.lookUpArray.length; i<ii; i++ ){

		if( walls.lookUpArray[ i ] ){

			// cycle through info
			for( var j=0, jj=cells.info.length; j<jj; j++ ){

				if( 
					( cells.info[ j ].faceIndex == walls.infoArray[ i ].faceIndex && cells.info[ j ].side == walls.infoArray[ i ].side ) ||
					( cells.list[ cells.info[ j ].faceIndex ].surroundingCells[ cells.info[ j ].side * 2 ] == walls.infoArray[ i ].faceIndex && sideReciprical[ cells.info[ j ].side ] == walls.infoArray[ i ].side )
				){

					cells.info.splice( j , 1 );
					jj = cells.info.length;
				}
			}
		}
	}
	// we'll do the same for each tower
	for( var i=0, ii=towers.length; i<ii; i++ ){

		for( var j=0, jj=towers[ i ].infoArray.length; j<jj; j++ ){

			for( var k=0, kk=cells.info.length; k<kk; k++ ){

				if( 
					( cells.info[ k ].faceIndex == towers[ i ].infoArray[ j ].faceIndex && cells.info[ k ].side == towers[ i ].infoArray[ j ].side ) ||
					( cells.list[ cells.info[ k ].faceIndex ].surroundingCells[ cells.info[ k ].side * 2 ] == towers[ i ].infoArray[ j ].faceIndex && sideReciprical[ cells.info[ k ].side ] == towers[ i ].infoArray[ j ].side ) ){

					cells.info.splice( k , 1 );
					kk = cells.info.length;
				}
			}
		}
	}

	updateCell( cells , true , "travelTo" );
}

mouseFunctions.updateCell = function( cells , bool , array ){

	updateCell( cells , bool , array );	
}

function updateCell( cells , bool , array ){

	var aCell, bCell, side;
	
	// the reciprical side, 0<->2 , 1<->3.
	var sideReciprical = [ 2 , 3 , 0 , 1 ];

	// -->> PROBABLY NOT NEEDED
	// we will change the passage directly and diagonally for each side
	// we'll record the diagonal index position in the surroundingCells array
	// side:0 has surroundingCells indexes 1,0 and 7.  etc...
	// REFER TO generateGameContent.createCells() for diagram.
	//var indexLookUp = [ { a:7 , b:1 } , { a:1 , b:3 } , { a:3 , b:5 } , { a:5 , b:7 } ];
	// -----<<

	// we will first set the direct passage from one cell to another and then vis-versa with the connect cells
	// passage back.
	// then we'll check for diagonal travel in the next for loop.
	
	for( var i=0, ii=cells.info.length; i<ii; i++ ){
		
		side = cells.info[ i ].side;

		aCell = cells.list[ cells.info[ i ].faceIndex ];
		aCell.currentIteration[ side * 2 ] = false;
		// -->> PROBABLY NOT NEEDED
		//aCell.currentIteration[ indexLookUp[ side ].a ] = false;
		//aCell.currentIteration[ indexLookUp[ side ].b ] = false;
		// -----<< 

		bCell = cells.list[ aCell.surroundingCells[ sideReciprical[ side ] * 2 ] ];
		bCell.currentIteration[ sideReciprical[ side ] * 2 ] = false;
		// -->> PROBABLY NOT NEEDED
		//bCell.currentIteration[ sideReciprical[ indexLookUp[ side ].a ] ] = false;
		//bCell.currentIteration[ sideReciprical[ indexLookUp[ side ].b ] ] = false;
		// -----<<
	}	

	// we need to update any cell that has this wall apart of it's surroundings
	// since we allow the AI to travel diagonally we have to check more conditions 
	// than if we only allowed left/right up/down movement.
	for( var i=0, ii=cells.info.length; i<ii; i++ ){

		switch( cells.info[ i ].side ){
			case 0:

				checkForDiagonalPassage( cells , i , array , 0 , 4 , 2 , 6 , 1 , 3 , 5 , 7 , bool );
				break;

			case 1:

				checkForDiagonalPassage( cells , i , array  , 2 , 6 , 0 , 4 , 1 , 7 , 5 , 3 , bool );
				break;

			case 2:

				checkForDiagonalPassage( cells , i , array  , 4 , 0 , 2 , 6 , 3 , 1 , 7 , 5 , bool );
				break;

			case 3:

				checkForDiagonalPassage( cells , i , array  , 6 , 2 , 0 , 4 , 7 , 1 , 3 , 5 , bool );
				break;
		}
	}
}

function checkForDiagonalPassage( cells , i , array , a , b , c , d , w , x , y , z , bool ){

	var aCell = cells.list[ cells.info[ i ].faceIndex ], bCell, cCell, dCell, eCell, fCell;

	aCell[ array ][ a ] = bool;

	bCell = cells.list[ aCell.surroundingCells[ a ] ];
	bCell[ array ][ b ] = bool;

	cCell = cells.list[ aCell.surroundingCells[ w ] ];
	dCell = cells.list[ aCell.surroundingCells[ z ] ];
	eCell = cells.list[ aCell.surroundingCells[ c ] ];
	fCell = cells.list[ aCell.surroundingCells[ d ] ];

	// check corners and look for a previous wall.  If one is found update all four cells for each
	// corner with false, otherswise, set the travelTo state to true.
	// 
	// 		| b | c |   |
	//		| a | x |   | 
	// 		|   |   |   |
	//
	// 		!( x -> a ) || !( a -> b ) || !( b -> c ){
	//			x->b && b->x && a->c && c->a = false	
	//		} else { 
	//			x->b && b->x && a->c && c->a = true	
	//		} 	

	if( !aCell[ array ][ a ] || !bCell[ array ][ c ] || !cCell[ array ][ b ] ){
		aCell[ array ][ w ] = false;
		bCell[ array ][ x ] = false;
		cCell[ array ][ y ] = false;
		eCell[ array ][ z ] = false;
	} else {
		aCell[ array ][ w ] = true;
		bCell[ array ][ x ] = true;
		cCell[ array ][ y ] = true;
		eCell[ array ][ z ] = true;

	}
	if( !aCell[ array ][ a ] || !bCell[ array ][ d ] || !dCell[ array ][ b ] ){
		aCell[ array ][ z ] = false;
		bCell[ array ][ y ] = false;
		dCell[ array ][ x ] = false;
		fCell[ array ][ w ] = false;
	} else {
		aCell[ array ][ z ] = true;
		bCell[ array ][ y ] = true;
		dCell[ array ][ x ] = true;
		fCell[ array ][ w ] = true;
	}
}

function iterateThroughAllDirections( cells , path ){

	var aCell, bCell, newPath=[];

	// the reciprical cell position of connected cell.
	var cellReciprical = [ 4 , 5 , 6 , 7 , 0 , 1 , 2 , 3 ];

	// cycle through all paths.
	for( var a=1, aa=path.length; a<aa; a++ ){

		aCell = cells.list[ path[ a ][ path[ a ].length - 1 ] ];

		//console.log( aCell );
		// we'll copy over the length of this path segment into the new array newPath.
		newPath = [ path[ a ][ 0 ] ];

		// we're going to store the length of the path in the zero index of newPath
		// if we travel from left/right or up/down we'll increment by 1
		// if diagonally, we'll increment by 1.415

		// check for passage to each of the eight connected cells
		for( var i=0; i<8; i++ ){
			
			bCell = cells.list[ aCell.surroundingCells[ i ] ];

			// if passage is allowed, copy the indexed path array and continue.
			if( aCell.currentIteration[ i ] ){

				newPath = [];

				for( var j=0, jj=path[ a ].length; j<jj; j++ ){
					newPath[ j ] = path[ a ][ j ];
				}

				// we'll add to the length of the current iteration/newPath
				if( i % 2 == 0 ){
					newPath[ 0 ] += 1;
				} else {
					newPath[ 0 ] += 1.415;
				}

				// adjust currentIteration for both to and from cells to false;
				aCell.currentIteration[ i ] = false;
				bCell.currentIteration[ cellReciprical[ i ] ] = false;

				//check to see if this cell is a goal cell
				if( bCell.goal ){

					// we've found a path to the exit.
					path[ 0 ].goal = true;

					// is this newly found path shorter than any previous ones?
					if( newPath[ 0 ] < path[ 0 ].length ){
						path[ 0 ].index = a;
						path[ 0 ].length = newPath[ 0 ];
					}

					// we'll cap this 'path' with the goals location
					path[ a ].push( aCell.surroundingCells[ i ] );
					i=8;
					
				} else {

					// we'll increment another time since we haven't reached the goal.
					newPath.push( aCell.surroundingCells[ i ] );
					path.push( newPath );

					// cause another cycle of the funciton.
					aa++;
				}
			} 
		}
	}
}