
var camera, scene, renderer, keyboard, lightSource;
var mouseVector2, mouseRay;
var grid, gridTargetMesh, cells, walls, towers, towerWalls, stageManager, shapeModels, currentStage;
var shapesToSpawn=[], shapes=[];
var userTaskManager, gameParameters, colorTemplate;
var cubemesh;
var shapeModelText;

// DEBUGGER VARIABLES
var counter=0;

init();
animate();

function init() {

	camera = new THREE.PerspectiveCamera( 20, 1024/768 , 0.01, 30 );
	camera.position.set( 2 , 2 , 2 );
	camera.lookAt( new THREE.Vector3( 0 , 0 , 0 ) );

	//we're going to add a feature to the camera allowing us to toggle the screen size when the user presses 'f'
	camera.renderSizeToggle = false;

	// any visual content we want displayed on screen we'll add to the scene.
	scene = new THREE.Scene();

	renderer = new THREE.WebGLRenderer( { antialias: true } );
	renderer.setSize( 1024 , 768 );
	document.body.appendChild( renderer.domElement );

	lightSource = new THREE.DirectionalLight( 0xffffff, 1 );
	lightSource.position.set( 1 , 2 , 3 );
	scene.add( lightSource );	

	// mouse controls
	document.addEventListener( 'mousedown' , mouseControls.checkMouseDown , false );	

	// keyboard controls
	keyboard = {};
	window.onkeyup = function(e) { keyboard[e.keyCode] = false; }
	window.onkeydown = function(e) { keyboard[e.keyCode] = true; }
	
	// we're going to add a timestamp to the keyboard allowing us to limit how fast the keyfire method fires under
	// certain circumstances.
	keyboard.timeStamp = +new Date();

	/*
	* game content 
	*
	*	first we'll add all the features for a playable game, 
	* 	then we'll add an html overlay to act as our start screen.
	*/

	// PART B
	// - create object to keep track of game parameters
	// - create object to keep track of user selected tasks
	// - create wall Mesh

	// create game parameters
	gameParameters = generateGameContent.createGameParameters();

	// create userSelectionTaskManager
	userTaskManager = generateGameContent.createUserSelectionTaskManager();

	// create a wall mesh which will serve as our visual representation of all walls
	// to save on processing time, we'll treat all walls as a single mesh, therefore we'll 
	// 'buffer' into it all vertices and faces necessary.  Then when a user creates a wall
	// we'll just update the positions of the vertices as needed.
	// for now we'll allow 40 wall segments @ 4 vertices per wall and 2 faces.
	walls = generateGameContent.generateWallMesh();
	scene.add( walls );

	// PART A
	// - create visual grid
	// - create target mesh
	// - create cell array to reference between the visual grid and target mesh, 
	//	 it will also act as our pathfinding array later on.


	// generate a grid and add it to the scene.
	grid = generateGameContent.generateGrid( gameParameters.gridColumns , gameParameters.gridRows );
	scene.add( grid );

	// since the grid will act as our visual reprentation of the playing area, we'll need to know
	// which square in the grid is being selected.  For that we'll add a grid target mesh.
	gridTargetMesh = generateGameContent.generateGridTargetMesh( grid );
	scene.add( gridTargetMesh );

	// create cells to act as the go between grid and gridTargetMesh, 
	// and they will also be used later on for pathfinding.
	cells = generateGameContent.createCells( grid , gridTargetMesh );

	// PART C
	// - create towers array
	// - cause tower placement to test for clear passage, only allow construction if passage is clear
	// - included tetraTower design.
	// - We're going to allow a total of 5 towers to be built.  Each tower constructed will have 8 walls 
	// 	 	surrounding it.  We'll create a towerWall buffer which will allow for 40 total walls to be built.
	// - create a dummy enemy to travel through the stage.
	// 		once the stage has been started, we'll prevent the creation of walls and towers.
	//		tower turrets will rotate to point at the dummy enemy.

	// tower array
	towers=[];

	// create Tower Walls
	towerWalls = generateGameContent.generateTowerWallMesh();
	scene.add( towerWalls );

	// PART D
	// - create wave manager that determines the order and timing of shapes being introduced to the playing field.
	// - create stage manager that will load a stage once 's' has been pressed.  The stage will start
	
	// - create enemy lexicon
	colorTemplate = generateGameContent.createColorTemplate();

	// load the text for each model.
	shapeLexicon = models.shapeLexicon();

	// construct stages
	stageManager = generateGameContent.createStages();

	// set current stage
	currentStage = 0;

}

function animate() {

	requestAnimationFrame( animate );

	keyboardControls.checkKeysPressed( keyboard );

	if( gameParameters.startWaveManager ){ 
		waveManager.currentWave();
	}

	renderer.render( scene, camera );

}

