var waveManager = waveManager || {};

/*
*	NOTE: Throughout WaveManager there is reference to shapes[ i ] which is the 3D mesh, and 
*			shapes[ i+1 ] which is the wireframe of the shape.
*			As each face of the shape is shot out, the face in question will disapper.  However,
*			the wireframe will remain visble until the shape has been broken.
*/

waveManager.currentWave = function(){

	// first we'll check to see if shapesToSpawn has any shapes left.
	if( shapesToSpawn.length > 0 ){

		// countdown timer to when shape should spawn.
		shapesToSpawn[ 0 ].timeToSpawn --;

		// if the timeToSpawn is 0, then we're going to add this shape over to
		// our shapes array which controls the shapes passage through the grid.
		if( shapesToSpawn[ 0 ].timeToSpawn == 0 ){

			// copy over the shape to the shapes array.
			shapes.push( shapesToSpawn[ 0 ] );
			shapes.push( shapesToSpawn[ 1 ] );

			// add them to the scene
			scene.add( shapes[ shapes.length - 2 ] );
			scene.add( shapes[ shapes.length - 1 ] );

			// remove this shape from shapesToSpawn
			shapesToSpawn.splice( 0 , 2 );
		}
	}

	// cycle through all shapes
	for( var i=0, ii=shapes.length; i<ii; i+=2 ){

		// check if the enemy needs to spawn.
		// if so, we'll select a starting location and process and path for it.
		if( shapes[ i ].spawn ){

			shapes[ i ].spawn = false;

			// assign a path for this shape to follow.
			shapes[ i ].path = getPath( cells );
			shapes[ i ].speed = 0.002;

			// we'll get the position for the shape.
			var position = cells.list[ shapes[ i ].path[ 0 ] ].center.clone();
			shapes[ i ].position.set( position.x , position.y , position.z );
			shapes[ i+1 ].position.set( position.x , position.y , position.z );

			// we're going to construct an incremental vector.  It'll be a normalized vector pointing in the 
			// direction of the next cell,  Then it's length will be divided by the shape's speed.
			// then each iteration of the shape's movement, we'll add the increment vector to it's position.
			var increment = cells.list[ shapes[ i ].path[1] ].center.clone();
			increment = increment.sub( position ).normalize();
			increment.multiplyScalar( shapes[ i ].speed );
			shapes[ i ].increment = increment.clone();

			// we'll record the next position
			shapes[ i ].next = 1;
			shapes[ i ].goal = cells.list[ shapes[ i ].path[1] ].center.clone();
			
			shapes[ i ].visible = true;
			shapes[ i+1 ].visible = true;

		} else if( shapes[ i ].broken ){

			// if the shape is broken we'll check to see if it's still visible
			if( shapes[ i ].visible ){

				// the shape will continue to move
				// but we'll animate it falling down and breaking up.
				shapes[ i ].position.set( shapes[ i ].position.x + shapes[ i ].increment.x , shapes[ i ].position.y - 0.0075 , shapes[ i ].position.z + shapes[ i ].increment.z );
				shapes[ i+1 ].position.set( shapes[ i ].position.x , shapes[ i ].position.y , shapes[ i ].position.z );

				// we're going to animate the breaking up part now
				// we need two place holder vectors
				var vectorOne, VectorTwo;

				for( var j=0, jj=shapes[ i+1 ].geometry.vertices.length; j<jj; j+=2 ){

					vectorOne = shapes[ i+1 ].geometry.vertices[ j ];
					vectorTwo = shapes[ i+1 ].geometry.vertices[ j+1 ];

					vectorOne.set( vectorOne.x + shapes[ i ].break[ j*0.5 ].x , vectorOne.y + shapes[ i ].break[ j*0.5 ].y , vectorOne.y + shapes[ i ].break[ j*0.5 ].y );
					vectorTwo.set( vectorTwo.x + shapes[ i ].break[ j*0.5 ].x , vectorTwo.y + shapes[ i ].break[ j*0.5 ].y , vectorTwo.y + shapes[ i ].break[ j*0.5 ].y );

				}

				shapes[ i+1 ].geometry.verticesNeedUpdate = true;				

				// if the shape is below -0.3 will turn off it's visibility,
				// and we'll remove it from the scene.
				if( shapes[ i+1 ].position.y < -0.3 ){
					shapes[ i ].visible = false;
					shapes[ i+1 ].visible = false;

					scene.remove( shapes[ i ] );
					scene.remove( shapes[ i+1 ] );
				}
			}

		} else {

			// we'll check to see if the shape has any pentagons to regenerate
			if( shapes[ i ].shapeIndex.regenerate ){
				
				shapes[ i ].shapeIndex.countDown ++;

				// we'll set the regeneration cycle to 60.
				if( shapes[ i ].shapeIndex.countDown == 60 ){

					regenerateFace( shapes[ i ] );

				}

			}

			// we'll move the shape
			shapes[ i ].position.set( shapes[ i ].position.x + shapes[ i ].increment.x , shapes[ i ].position.y + shapes[ i ].increment.y , shapes[ i ].position.z + shapes[ i ].increment.z );
			shapes[ i+1 ].position.set( shapes[ i ].position.x , shapes[ i ].position.y , shapes[ i ].position.z );

			// and rotate the shape
			shapes[ i ].rotateY( 0.03 );
			shapes[ i+1 ].rotateY( 0.03 );

			// now we'll test to see if the distance between the dummies position and it's goal is the almost zero
			if( shapes[ i ].position.distanceTo( shapes[ i ].goal ) < shapes[ i ].speed ){

				shapes[ i ].next ++;

				// now we'll test if we've come to the end.  If so we'll reset the shape back to the beginning.
				if( shapes[ i ].next > shapes[ i ].path.length-1 ){

					// we'll get the position for the shape.
					var position = cells.list[ shapes[ i ].path[ 0 ] ].center.clone();
					shapes[ i ].position.set( position.x , position.y , position.z );
					shapes[ i+1 ].position.set( position.x , position.y , position.z );

					// we're going to construct an incremental vector.  It'll be a normalized vector pointing in the 
					// direction of the next cell,  Then it's length will be divided by the shape's speed.
					// then each iteration of the shape's movement, we'll add the increment vector to it's position.
					var increment = cells.list[ shapes[ i ].path[1] ].center.clone();
					increment = increment.sub( position ).normalize();
					increment.multiplyScalar( shapes[ i ].speed );
					shapes[ i ].increment = increment.clone();

					// we'll record the next position
					shapes[ i ].next = 1;
					shapes[ i ].goal = cells.list[ shapes[ i ].path[1] ].center.clone();

				} else {

					shapes[ i ].goal = cells.list[ shapes[ i ].path[ shapes[ i ].next ] ].center.clone();
					// we'll increment to the next position in the path.
					var position = shapes[ i ].position.clone();

					// we're going to construct an incremental vector.  It'll be a normalized vector pointing in the 
					// direction of the next cell,  Then it's length will be divided by the shape's speed.
					// then each iteration of the shape's movement, we'll add the increment vector to it's position.
					var increment = cells.list[ shapes[ i ].path[ shapes[ i ].next ] ].center.clone();
					increment = increment.sub( position ).normalize();
					increment.multiplyScalar( shapes[ i ].speed );
					shapes[ i ].increment = increment.clone();
				}
			}
		}
	}

	// cycle through all viewable towers and orient their turrets toward the targeted shape
	var Tower, Skip;
	for( var i=0, ii=towers.length; i<ii; i++ ){

		Tower = towers[ i ].turret;

		// we'll increment the rest of the tower.
		Tower.details.rest ++;

		if( Tower.details.animating ){

			// if the tower is in the middle of an animation cycle we won't update it's lookAt position
			// instead we'll let the tower finish its animation cycle before we update the rotation of the turret.
			// we'll also process the hit for the enemy.

			animations.animateTower( Tower , towers[ i ].type , shapes[ Tower.details.targetShape ] );

		} else {

			// if we find the existing target isn't broken and still within range, we'll skip a later step.
			Skip = false;

			// we'll check to see if this tower has a shape that it's already been targeting
			if( Tower.details.targetShape > -1 ){

				// check to see if the shape isn't broken, and if it's still within range of the tower
				if(	shapes[ Tower.details.targetShape ].position.distanceToSquared( Tower.position ) <= Tower.details.targetRadius[ Tower.details.level ] &&
					!shapes[ Tower.details.targetShape ].broken ){

					// since the same target is not broken yet, and still within range we can skip the
					// the step looking for a target for the tower.
					
					Skip = true;

					Tower.lookAt( shapes[ Tower.details.targetShape ].position.clone() );

					//check to see if the rest is equal to the next fire schedule.
					if( Tower.details.rest >= Tower.details.fireSchedule[ Tower.details.level ] ){

						// this tower is ready to fire, no we have to process the preanimation calculations.
						// each tower is different in this respect.  
						animations.animationPresets( Tower , towers[ i ].type );
						
					}
				} 	
			}

			if( !Skip ){
				// check to see if a shape is within the towers target radius
				// for now we'll retrieve the first on our list
				for( var j=0, jj=shapes.length; j<jj; j+=2 ){

					// check if this shape is broken, but in the broken animation process.
					if( !shapes[ j ].broken ){

						// check to see if the shape is within the towers range
						if( shapes[ j ].position.distanceToSquared( Tower.position ) <= Tower.details.targetRadius[ Tower.details.level ] ){

							// this shape has been targeted.
							// we'll add this shapes index position to the Tower
							Tower.details.targetShape = j;

							Tower.lookAt( shapes[ j ].position.clone() );

							//check to see if the rest is equal to the next fire schedule.
							if( Tower.details.rest >= Tower.details.fireSchedule[ Tower.details.level ] ){

								// this tower is ready to fire, no we have to process the preanimation calculations.
								// each tower is different in this respect.  
								animations.animationPresets( Tower , towers[ i ].type );
								
							}
						}
					}
				}
			}
		}
	}
}

function regenerateFace( shape ){

	shape.shapes.pentagons +=3;
	shape.shapes.total +=3;

	var faceOne = shape.geometry.faces[ shape.shapeIndex.pentagonsToRegernerate[ 0 ] ];
	var faceTwo = shape.geometry.faces[ shape.shapeIndex.pentagonsToRegernerate[ 1 ] ];
	var faceThree = shape.geometry.faces[ shape.shapeIndex.pentagonsToRegernerate[ 2 ] ];

	// restore the original draw order
	faceOne.a = faceOne.oldA;	faceTwo.a =	faceTwo.oldA;	faceThree.a = faceThree.oldA;
	faceOne.b = faceOne.oldB;	faceTwo.b = faceTwo.oldB;	faceThree.b = faceThree.oldB;
	faceOne.c = faceOne.oldC;	faceTwo.c = faceTwo.oldC;	faceThree.c = faceThree.oldC;

	shape.geometry.elementsNeedUpdate = true;

	// remove these three faces from the pentagons array and add them to
	// the pentagonsToRegernerate array
	shape.shapeIndex.pentagons.push( shape.shapeIndex.pentagonsToRegernerate[ 0 ] );
	shape.shapeIndex.pentagons.push( shape.shapeIndex.pentagonsToRegernerate[ 1 ] );
	shape.shapeIndex.pentagons.push( shape.shapeIndex.pentagonsToRegernerate[ 2 ] );

	shape.shapeIndex.pentagonsToRegernerate.splice( 0 , 3 );

	if( shape.shapeIndex.pentagonsToRegernerate.length > 0 ){
		shape.shapeIndex.regenerate = true;
	} else {
		shape.shapeIndex.regenerate = false;
	}

	shape.shapeIndex.countDown = 0;
}

function getPath( cells ){

	var aCell;

	// set all currentIteration arrays to their travelTo state.
	for( var i=0, ii=cells.list.length; i<ii; i++ ){
		aCell = cells.list[ i ];
		for( var j=0; j<8; j++ ){
			aCell.currentIteration[ j ] = aCell.travelTo[ j ];
		}
	}

	// path will be an array that we add to.  At each iteration of our Flood Fill pathfinding algorithm
	// the path will copy its self for each subsequent iteration and add a unique bit to each new construct.  
	// Each Path item will contain an array of all spaces it has traveled to and begin with the overall distance.
	// Each enemy will spawn on a random startIndex cell contained in the cells.startIndexes array.
	// the first entry in the path array will be an object indicating whether or not an end goal has been found.
	// and what the indexed position is for the shortest path and the length of the shortest path.
	var randomStartingIndex =  cells.startingIndexes[ Math.floor( Math.random() * cells.startingIndexes.length ) ];
	//console.log( randomStartingIndex );
	var path = [ { goal: false , index: -1, length: 10000 } , [ 0 , randomStartingIndex ] ];

	iterateThroughAllDirections( cells , path );

	// update the length of the path to reflect the actual length.
	path[ path[0].index ].splice( 0 , 1 );

	// we'll return the path which is the shortest, which is recorded in the index position
	return path[ path[0].index ];
}